package com.gct.edge.zdly.aoid.core;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.edge.entity.AoidConfigData;
import com.gct.edge.service.abst.AoidConfigDataService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author xusirui 2024/3/22
 */
@Component
@Slf4j
public class GlobalDataManager implements InitializingBean {

    private final static String STRING_TYPE = "STRING_TYPE";
    private final static String DOUBLE_TYPE = "DOUBLE_TYPE";
    private final static String INTEGER_TYPE = "INTEGER_TYPE";

    @Value("${gct.base-data.global-config-path}")
    String globalConfigPath;

    @Value("${gct.base-data.static-data-path}")
    String staticDataPath;

    @Autowired
    AoidConfigDataService aoidConfigDataService;

    private Map<String, Object> globalDataMap = new ConcurrentHashMap<>();

    public void setGlobalData(String key, Object val) {
        globalDataMap.put(key, val);
    }

    public <T> T getGlobalData(String key, Class<T> typeClass) {
        return (T) globalDataMap.get(key);
    }

    public void refreshData() {
        refreshGlobalData();
        refreshAoidConfigData();
    }

    private void refreshGlobalData() {
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(globalConfigPath)));
            String line;
            while ((line = br.readLine()) != null) {
                if (line.startsWith("#") || line.startsWith("[") || !line.contains("=")) continue;
                String[] split = line.split("=");
                String key = split[0].trim();
                String val = split[1].trim();
                if (val.contains(" ") || val.contains(",")) {
                    String splitTag = val.contains(" ") ? " " : ",";
                    String[] vals = val.split(splitTag);
                    boolean isString = Arrays.stream(vals).anyMatch(x -> checkValType(x).equals(STRING_TYPE));
                    if (isString) {
                        this.setGlobalData(key, Arrays.asList(vals));
                    } else {
                        boolean isDouble = Arrays.stream(vals).anyMatch(x -> checkValType(x).equals(DOUBLE_TYPE));
                        if (isDouble) {
                            List<Double> valList = Arrays.stream(vals).map(Double::parseDouble).collect(Collectors.toList());
                            this.setGlobalData(key, valList);
                        } else {
                            List<Integer> valList = Arrays.stream(vals).map(Integer::parseInt).collect(Collectors.toList());
                            this.setGlobalData(key, valList);
                        }
                    }
                } else {

                    String valType = checkValType(val);
                    if (valType.equals(STRING_TYPE)) {
                        this.setGlobalData(key, val);
                    } else if (valType.equals(DOUBLE_TYPE)) {
                        this.setGlobalData(key, Double.parseDouble(val));
                    } else {
                        this.setGlobalData(key, Integer.parseInt(val));
                    }
                }
            }
        } catch (Exception e) {
            log.error("load global-data from {} error, detail: {}", globalConfigPath, e.getMessage());
        }
    }

    private void refreshAoidConfigData() {
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(staticDataPath)));
            String line;
            List<AoidConfigData> configDataList = new ArrayList<>();
            boolean isValid = true;
            while ((line = br.readLine()) != null) {
                String[] split = line.split(",");
                if (split.length != 8) {
                    log.error("aoid_config_data文件每列必须包含八个配置项, {}", line);
                    isValid = false;
                    continue;
                }
                AoidConfigData config = new AoidConfigData();
                config.setJh(split[0].trim());
                config.setBj(split[1].trim());
                config.setBs(split[2].trim());
                config.setHsl(split[3].trim());
                config.setSmd(split[4].trim());
                config.setYmd(split[5].trim());
                config.setQyb(split[6].trim());
                config.setDym(split[7].trim());
                configDataList.add(config);
            }
            if (isValid && !configDataList.isEmpty()){
                aoidConfigDataService.remove(new QueryWrapper<AoidConfigData>().last(" where 1 = 1"));
                for (AoidConfigData config : configDataList) {
                    aoidConfigDataService.save(config);
                }
            }
        } catch (Exception e) {
            log.error("load aoid-config-data from {} error, detail: {}", staticDataPath, e.getMessage());
        }
    }

    private boolean checkNonNumber(String str) {
        String regex = ".*[a-zA-zZ]+.*";
        return Pattern.compile(regex).matcher(str).matches();
    }

    private String checkValType(String val) {
        for (int i = 0; i < val.getBytes().length; i++) {
            if (!(Character.isDigit(val.getBytes()[i]) || val.getBytes()[i] == '.')) return STRING_TYPE;
        }
        if (val.contains(".") && val.indexOf(".") == val.lastIndexOf(".")) return DOUBLE_TYPE;
        if (!val.contains(".")) return INTEGER_TYPE;
        return STRING_TYPE;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        refreshData();
    }
}
