package com.gct.edge.zdly.dgns.thriftclient;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.thrift.TException;
import org.apache.thrift.protocol.TBinaryProtocol;
import org.apache.thrift.transport.TSocket;
import org.apache.thrift.transport.TTransport;
import org.apache.thrift.transport.TTransportException;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.io.Closeable;
import java.util.List;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 * dec:
 * <p>
 * EvaluationServerWrapper
 * </p>
 *
 * @author lloyd
 * @version 1.0.1
 * @since 2021/12/9 下午4:33
 */
@Slf4j
@Component
public class EvaluationServerWrapper implements EvaluationServer.Iface, Closeable, DisposableBean, Runnable {

    private static ConcurrentHashMap<UUID, Object> responseDataBuffer = new ConcurrentHashMap<>();
    private static ConcurrentLinkedQueue<RequireData> requireDataBuffer = new ConcurrentLinkedQueue();

    private static Object waitKey = new Object();
    private EvaluationServer.Client client = null;
    private TTransport transport = null;

    ThriftClientProperties config;

    ThriftSocketClientPoolProperties poolConfig;

    private void setClient(EvaluationServer.Client client) {
        this.client = client;
    }

    private void setTTransport(TTransport transport) {
        this.transport = transport;
    }

    /**
     * flag for client if available
     */
    private final ThreadLocal<Boolean> available = new ThreadLocal<>();

    public EvaluationServerWrapper(ThriftClientProperties config) {
        this.config = config;
    }

    /**
     * @return connected return true or return false
     */
    public boolean connect() {
        if (Objects.nonNull(available.get()) && available.get()) return true;//avoid repeat initialize the connection
        disConnect();
        if (Objects.isNull(config) || StringUtils.isEmpty(config.getHost()) || StringUtils.isEmpty(config.getPort()) || StringUtils.isEmpty(config.getTimeout())) {
            log.error("The config.get() config is invalid.");
            available.set(false);
            return false;
        }
        setTTransport(new TSocket(config.getHost(), config.getPort(), config.getTimeout()));
        setClient(new EvaluationServer.Client(new TBinaryProtocol(transport)));
        try {
            transport.open();
            available.set(true);
            return true;
        } catch (TTransportException ignore) {
            log.warn("Can not connect the server by config:{}", JSONObject.toJSONString(config));
            available.set(false);
            return false;
        }
    }


    @Override
    public int healthyCheck() {
//        if (!connect()) return -1;

        try {
            return client.healthyCheck();
        } catch (TException e) {
            available.set(false);
            return -2;
        }
    }

    @Override
    public boolean accessApply(String key, String model_version, String model_path, String type_mapper_path) {
//        if (!connect()) return false;

        try {
            return client.accessApply(key, model_version, model_path, type_mapper_path);
        } catch (TException e) {
            available.set(false);
            return false;
        }
    }

    @Override
    public int predictId(String key, String model_version, List<List<Double>> diagram) {
//        if (!connect()) return -1;

        try {
            return client.predictId(key, model_version, diagram);
        } catch (TException e) {
            available.set(false);
            return -2;
        }
    }

    @Override
    public String predictCategory(String key, String model_version, List<List<Double>> diagram) {
//        if (!connect()) return null;

        try {
            return client.predictCategory(key, model_version, diagram);
        } catch (TException e) {
            available.set(false);
            return null;
        }
    }

    public int asyncPredictIdWithDefault(List<List<Double>> diagram) {
        RequireData args = new RequireData().setHandler(UUID.randomUUID()).setArgs(new Object[]{config.getKey(), diagram});
        requireDataBuffer.add(args);
        synchronized (waitKey) {
            waitKey.notify();
        }
        return (int) getResponse(args.handler);
    }

    private void registerPredictIdWithDefaultHandler() {
        while (true) {
            RequireData one = requireDataBuffer.poll();
            if (Objects.isNull(one))
                synchronized (waitKey) {
                    try {
                        waitKey.wait();
                    } catch (InterruptedException e) {
                        disConnect();
                        log.info("{}:已退出",Thread.currentThread().getName());
                    }
                    continue;
                }
            int id = predictIdWithDefault((String) one.args[0], (List<List<Double>>) one.args[1]);
            responseDataBuffer.put(one.getHandler(), id);
        }
    }

    private static Object getResponse(UUID handler) {
        while (true) {
            Object val = responseDataBuffer.get(handler);
            if (Objects.nonNull(val)) return val;
        }
    }

    @Override
    public int predictIdWithDefault(String key, List<List<Double>> diagram) {
        if (!connect()) return -1;

        try {
            return client.predictIdWithDefault(key, diagram);
        } catch (TException e) {
            e.printStackTrace();
            available.set(false);
            return -2;
        }
    }

    @Override
    public String predictCategoryWithDefault(String key, List<List<Double>> diagram) {
//        if (!connect()) return null;

        try {
            return client.predictCategoryWithDefault(key, diagram);
        } catch (TException e) {
            available.set(false);
            return null;
        }
    }

    @Override
    public double similarity(String key, List<List<Double>> base, List<List<Double>> match) {
//        if (!connect()) return -2;

        try {
            return client.similarity(key, base, match);
        } catch (TException e) {
            available.set(false);
            return -1;
        }
    }

    public void disConnect() {
        try {
            if (Objects.nonNull(transport) && transport.isOpen()) transport.close();
        } catch (Exception ignore) {
        }
        client = null;
        transport = null;
    }

    @Override
    public void close() {
        disConnect();
    }

    @Override
    public void destroy() {
        close();
    }

    @Override
    public void run() {
        connect();
        registerPredictIdWithDefaultHandler();
    }
}
