package com.gct.edge.zdly.dgns.thriftclient;


import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.stereotype.Component;

import java.util.LinkedList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * <p>
 * ThriftClient: for diagram_card recognition
 * <p>
 * use @Autowired to inject your dependence location
 * </p>
 *
 * @author lloyd
 * @version 1.0
 * @since 2021.1.21
 */
@Slf4j
@Component
@EnableConfigurationProperties({ThriftSocketClientPoolProperties.class})
public class GTComputerViewRecognitionClient implements DisposableBean {
    private static final int DEFAULT_PREDICT_TYPE = -1;
    private static final double DEFAULT_SIMILAR = -1;

    //    private final ThriftClientFactory factory=null;
//    gtcvrService.Client client;
    EvaluationServerWrapper client;

    ReadWriteLock lockClient = new ReentrantReadWriteLock();

    /*@Autowired
    public GTComputerViewRecognitionClient(ThriftClientFactory factory) {
        this.factory = factory;
        new Thread(() -> {
            */
    /**
     * 0.5s检测一下链接，重连
     *//*
            while (true) {
                refreshClient();
                try {
                    Thread.sleep(500l);
                } catch (InterruptedException e) {
                }
            }
        }).start();

    }*/
    private final ThriftSocketClientPoolProperties poolProperties;

    private static final LinkedList<Thread> poolObjects=new LinkedList<>();
    public GTComputerViewRecognitionClient(ThriftSocketClientPoolProperties poolProperties, ThriftClientProperties clientProperties, EvaluationServerWrapper client) {
        if (Objects.isNull(poolProperties) || poolProperties.getMaxTotal() <= 0)
            throw new ValidateFailureClient("gtcvr连接池配置错误,config:=>"+ JSON.toJSONString(poolProperties));
        this.client= client;
        this.poolProperties = poolProperties;
        for (int i = 0; i < poolProperties.getMaxTotal(); i++) {
            Thread thread = new Thread(new EvaluationServerWrapper(client.config));
            poolObjects.add(thread);
            thread.start();
        }
        log.info("gtcvr 连接池配置成功,size:=>{}",poolProperties.getMaxTotal());
    }

    private void refreshClient() {

        try {
            boolean lock = lockClient.writeLock().tryLock(100l, TimeUnit.MILLISECONDS);
            if (!lock) return;
        } catch (InterruptedException e) {
//            e.printStackTrace();
            return;
        }
        try {
//            client = factory.getClient();
            client.connect();
        } catch (Exception e) {

        } finally {
            lockClient.writeLock().unlock();
        }
    }

    private int GTPredictType(List<List<Double>> sgt, int sgtPointNum) {
        int result = DEFAULT_PREDICT_TYPE;
        lockClient.readLock().lock();
        try {
            result = client.predictIdWithDefault(client.config.getKey(), sgt);
        } catch (Exception e) {
            log.warn("RPC:ERROR:GTPredictType,cause by: {},msg: {}", e.getCause(), e.getMessage());
        } finally {
            lockClient.readLock().unlock();
        }
        return result;
    }

    private double SGTSimilarByGrayVal(List<List<Double>> baseSGT, int baseSGTPointNum, List<List<Double>> matchSGT, int matchSGTPointNum) {
        double result = DEFAULT_SIMILAR;
        lockClient.readLock().lock();
        try {
            result = client.similarity(client.config.getKey(), baseSGT, matchSGT);
        } catch (Exception e) {
            log.warn("RPC:ERROR:SGTSimilarByGrayVal,cause by: {},msg: {}", e.getCause(), e.getMessage());
        } finally {
            lockClient.readLock().unlock();
        }
        return result;
    }

    /**
     * 进行功图图形识别
     *
     * @param sgt         功图
     * @param sgtPointNum 功图点数
     * @return ResponseData
     */
    public synchronized ThriftClientResponseData<Integer> tryGTPredictType(List<List<Double>> sgt, int sgtPointNum) {
        int pType = GTPredictType(sgt, sgtPointNum);
        if (pType == DEFAULT_PREDICT_TYPE) {
            return new ThriftClientResponseData<Integer>().setData(null).setSuccess(false);
        } else return new ThriftClientResponseData<Integer>().setData(pType).setSuccess(true);
        /*int failedCount = 0;
        do {
            int pType = GTPredictType(sgt, sgtPointNum);
            if (pType == DEFAULTPREDICTTYPE) {
                try {
                    Thread.sleep(50);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                return new ThriftClientResponseData<Integer>().setData(pType).setSuccess(true);
            }
            refreshClient();
        } while (failedCount++ < 5);
        return new ThriftClientResponseData<Integer>().setData(null).setSuccess(false);*/
    }

    /**
     * 进行功图图形识别 <br/>
     * 异步连接池无锁版
     *
     * @param sgt 功图
     * @return ResponseData
     */
    public ThriftClientResponseData<Integer> tryGTPredictType(List<List<Double>> sgt) {
        int pType = client.asyncPredictIdWithDefault(sgt);
        if (pType == DEFAULT_PREDICT_TYPE) {
            return new ThriftClientResponseData<Integer>().setData(null).setSuccess(false);
        } else return new ThriftClientResponseData<Integer>().setData(pType).setSuccess(true);
        /*int failedCount = 0;
        do {
            int pType = GTPredictType(sgt, sgtPointNum);
            if (pType == DEFAULTPREDICTTYPE) {
                try {
                    Thread.sleep(50);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                return new ThriftClientResponseData<Integer>().setData(pType).setSuccess(true);
            }
            refreshClient();
        } while (failedCount++ < 5);
        return new ThriftClientResponseData<Integer>().setData(null).setSuccess(false);*/
    }

    /**
     * 灰度计算两幅功图的特征相似度
     *
     * @param baseSGT          功图
     * @param baseSGTPointNum  功图点数
     * @param matchSGT         功图
     * @param matchSGTPointNum 功图点数
     * @return ResponseData
     */
    public synchronized ThriftClientResponseData<Double> trySGTSimilarByGrayVal(List<List<Double>> baseSGT, int baseSGTPointNum, List<List<Double>> matchSGT, int matchSGTPointNum) {
        double pSimilar = SGTSimilarByGrayVal(baseSGT, baseSGTPointNum, matchSGT, matchSGTPointNum);
        if (pSimilar == DEFAULT_PREDICT_TYPE)
            return new ThriftClientResponseData<Double>().setData(null).setSuccess(false);
        else return new ThriftClientResponseData<Double>().setData(pSimilar).setSuccess(true);
    }

    public int healthyCheck() {
        int result = 0;
        lockClient.readLock().lock();
        try {
            result = client.healthyCheck();
        } catch (Exception e) {
            log.warn("RPC:ERROR:SGTSimilarByGrayVal");
        } finally {
            lockClient.readLock().unlock();
        }
        return result;
    }

    @Override
    public void destroy() throws Exception {
        for (Thread poolObject : poolObjects) {
            poolObject.interrupt();
        }
        log.info("gtcvr连接池已销毁");
    }
}
